@@ -539,12 +539,18 @@ load_menu_from_directory (LauncherMenu *menu,
         }
 
         break;
+
+      case GMENU_TREE_ITEM_SEPARATOR:
+        gmenu_tree_item_unref (item);
+        goto out;
         
       default:
         break;
     }
     gmenu_tree_item_unref (item);
   }  
+out:
+
   g_slist_free (list);
 }
 
@@ -663,84 +669,6 @@ free_menu (LauncherMenu *menu)
   i++;
 }
 
-static gboolean
-list_changed (LauncherMenu *menu)
-{
-  LauncherMenuPrivate *priv;
-  GMenuTreeDirectory *root;
-  
-  g_return_val_if_fail (LAUNCHER_IS_MENU (menu), FALSE);
-  priv = menu->priv;
-
-  g_debug ("menu changed\n");
-
-  priv->old_cats = priv->categories;
-  priv->old_apps = priv->menu_apps;
-  priv->categories = priv->menu_apps = NULL;
-
-  priv->refresh = TRUE;
-
-  root = gmenu_tree_get_root_directory (priv->app_tree);
-  load_menu_from_directory (menu, root, NULL);
-  gmenu_tree_item_unref (root);
-
-  priv->refresh = FALSE;
-
-  free_menu (menu);
-
-  /*
-  root = gmenu_tree_get_root_directory (priv->sys_tree);
-  load_menu_from_directory (menu, root, NULL);
-  gmenu_tree_item_unref (root);
-  */
-  g_signal_emit (menu, _menu_signals[MENU_CHANGED], 0);
-  menu->priv->tag = 0;  
-  
-  return FALSE;
-}
-
-/*
- * FIXME: The tree changing should not be destructive. Find a way to make it
- * work without re-creating the entire menu again.
- */
-static void
-tree_changed (GMenuTree *tree, LauncherMenu *menu)
-{
-  LauncherMenuPrivate *priv;
-  GMenuTreeDirectory *root;
-  
-  g_return_if_fail (LAUNCHER_IS_MENU (menu));
-  priv = menu->priv;
-
-  root = gmenu_tree_get_root_directory (tree);
-  gmenu_tree_item_unref (root);  
-  
-  if (priv->tag)
-  {
-    return;
-  }
-  priv->tag = g_timeout_add (500, (GSourceFunc)list_changed, menu);
-}
-
-static void
-on_theme_changed (GtkIconTheme *theme, LauncherMenu *menu)
-{
-  LauncherMenuPrivate *priv;
-  GMenuTreeDirectory *root;
-  
-  g_return_if_fail (LAUNCHER_IS_MENU (menu));
-  priv = menu->priv;
-
-  root = gmenu_tree_get_root_directory (priv->app_tree);
-  gmenu_tree_item_unref (root);  
-  
-  if (priv->tag)
-  {
-    return;
-  }
-  priv->tag = g_timeout_add (500, (GSourceFunc)list_changed, menu);
-}
-
 static void
 sort_categories (LauncherMenu *menu)
 {
@@ -791,6 +719,97 @@ sort_categories (LauncherMenu *menu)
   sort_categories (menu);
 }
 
+static gboolean
+list_changed (LauncherMenu *menu)
+{
+  LauncherMenuPrivate *priv;
+  GMenuTreeDirectory *root;
+  GList *l, *fav = NULL;
+  
+  g_return_val_if_fail (LAUNCHER_IS_MENU (menu), FALSE);
+  priv = menu->priv;
+
+  g_debug ("Menu changed");
+
+  priv->old_cats = priv->categories;
+  priv->old_apps = priv->menu_apps;
+  priv->categories = priv->menu_apps = NULL;
+
+  priv->refresh = TRUE;
+
+  root = gmenu_tree_get_root_directory (priv->app_tree);
+  load_menu_from_directory (menu, root, NULL);
+  gmenu_tree_item_unref (root);
+
+  priv->refresh = FALSE;
+
+  free_menu (menu);
+
+  for (l = priv->categories; l; l= l->next)
+  {
+    LauncherMenuCategory *cat = l->data;
+    if (cat->icon && strstr (cat->icon, FAVCATNAME))
+    {
+      fav = l;
+      break;
+    }
+  }
+  if (fav)
+  {
+    LauncherMenuCategory *favcat = fav->data;
+    priv->categories = g_list_delete_link (priv->categories, fav);
+    priv->categories = g_list_prepend (priv->categories, favcat);
+  }
+
+  g_signal_emit (menu, _menu_signals[MENU_CHANGED], 0);
+  menu->priv->tag = 0;  
+  
+  return FALSE;
+}
+
+/*
+ * FIXME: The tree changing should not be destructive. Find a way to make it
+ * work without re-creating the entire menu again.
+ */
+static void
+tree_changed (GMenuTree *tree, LauncherMenu *menu)
+{
+  LauncherMenuPrivate *priv;
+  GMenuTreeDirectory *root;
+  
+  g_return_if_fail (LAUNCHER_IS_MENU (menu));
+  priv = menu->priv;
+
+  root = gmenu_tree_get_root_directory (tree);
+  gmenu_tree_item_unref (root);  
+  
+  if (priv->tag)
+  {
+    return;
+  }
+  priv->tag = g_timeout_add (500, (GSourceFunc)list_changed, menu);
+}
+
+static void
+on_theme_changed (GtkIconTheme *theme, LauncherMenu *menu)
+{
+  LauncherMenuPrivate *priv;
+  GMenuTreeDirectory *root;
+  
+  g_return_if_fail (LAUNCHER_IS_MENU (menu));
+  priv = menu->priv;
+
+  root = gmenu_tree_get_root_directory (priv->app_tree);
+  gmenu_tree_item_unref (root);  
+  
+  if (priv->tag)
+  {
+    return;
+  }
+  priv->tag = g_timeout_add (500, (GSourceFunc)list_changed, menu);
+}
+
+
 /* GObject functions */
 static void
 launcher_menu_dispose (GObject *object)
@@ -846,7 +865,7 @@ launcher_menu_init (LauncherMenu *menu)
   priv->tag = 0;
   priv->refresh = FALSE;
 
-  priv->app_tree = load_menu_from_tree (menu, PKGDATADIR"/applications.menu");
+  priv->app_tree = load_menu_from_tree (menu, "applications.menu");
 
   sort_categories (menu);
 
